//LADOWARKA LI-ION 18650 v.2013-01
//
//
//uC ATMEGA 8
//
//FUSES (fuse bity):
//fusebit: 3F (BOD 4.0V, Ext Crystal / High Freq)
//fusebit high: D9
//
//
#include	<inttypes.h>
#include 	<avr/io.h>
#include 	<avr/iom8.h>
#include <avr/wdt.h> 
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include 	<util/delay.h>   

#include "../common/bascom_typy.h"
#include "../common/uart3.h"

#include "wysw3cy1sw.h" //obsluga wyswietlacza

//definicje dot. faz pracy
#define C_min_prad_ladowania 9 		// x10mA (tryby 1000mA i 500mA) - prad przy ktorym ladowanie zostanie uznane za zakonczone 
#define C_min_prad_ladowania200 3 		// x10mA (tryb 200mA) - prad przy ktorym ladowanie zostanie uznane za zakonczone 
#define C_zwloka_przed_pomiarem 30 		// x1s   - wyjasnione przy zmiennej timer_zwloka
#define C_max_napiecie_ladowania 420 // x10mV - maksymalne napiecie na zaciskach ogniwa podczas ladowania
#define C_napiecie_doladowania 412 // x10mV - napiecie jakie bedzie utrzymywane na ogniwie po zakonczonym cyklu ladowania
#define C_prad_doladowania 20 		// x10mA - maksymalny prad ogniwa po zakonczeniu ladowania  (w normalnych warunkach ten prad nie jest nigdy osiagany)
#define C_min_prad_awaria 1 			// x10mA - osiagniecie tego pradu lub mniejszego uznawane jest za awarie (rozwarcie itp)
#define C_min_napiecie_rozladowania 290 // x10mV - dolne napiecie do jakiego bedzie rozladowane badane ogniwo
#define C_kalibracja_korekta_pradu_ladowania 3 //x10mA - stala pozwala odliczyc prad zasilania ukladu od pradu ladowania


//definicje
#ifndef F_CPU
	#define F_CPU 12000000
#endif
#define UART_BAUD 9600
#define __UBRR F_CPU/16/UART_BAUD-1  // obliczamy UBRR dla U2X=0
#define C_stala_czasowa 3 //wartosc 'kondensatora' filtru LP
#define C_minuta 60 //liczba sekund w minucie
#define C_deltau 25 //poczatkowa wartosc kroku przy dostrajaniu PWM regulacji napiecia
#define C_PWMUstart 90 //poczatkowa wartosc PWM napiecia przy starcie ladowania (regulacja zacznie sie od okolo 3V)

//makra
#define PWMIRozladOn PORTC |= (1<<PC2)
#define PWMIRozladOff PORTC &= ~(1<<PC2)
#define PWMULadOn PORTC |= (1<<PC1)
#define PWMULadOff PORTC &= ~(1<<PC1)
#define str(s) #s  		//przeksztalcanie stalej liczbowej w znakowa
#define xstr(s) str(s)	//c.d.  uzycie np: Print(xstr(nazwa_stalej_doslownej_liczbowej));


//definicje A/D
#define PinPomiarILad 0
#define PinPomiarIRozlad 6
#define PinPomiarUBat 7


//deklaracje funkcji
void wysw_liczbe(word);
void kalibracja(void);
void PrintBuf_PROGMEM(char *); 
void PrintBuf_LICZBA_WORD(word, byte);
void podaj_parametry_testuj(void);
void podaj_parametry_laduj(void);

//deklaracaje zmiennych 
volatile struct {
	uint8_t led1:1;
	uint8_t led2:1;
	uint8_t led3:1;
	uint8_t migajled3:1;
	uint8_t migajwysw:1;
	uint8_t sw:1;
	uint8_t watch:1; //flaga dla watchdoga, ustawiana w przerwaniu 
	uint8_t flaga_pom:1; //flaga pomocnicza, zapewnia wyslanie komunikatu o koncu procesu tylko raz
} bityA;
volatile word etimer1;  //czas  zwiekszane co 125 us
volatile byte etimer2; //sekundnik wysylanie danych prze uart
volatile byte etimer3; //sekundnik stabilizacji pradu (sprawdzanie i korekta co sekunde)

volatile byte etimeradc; //czas zwiekszane co 32ms
volatile byte timer_zmiany; //odlicza czas wyswietlania nazwytrybu
byte wyswietlanie; //okresla co jest ukazywane aktualnie na wyswietlaczu
byte timer_zwloka; //zapewnia zwloke potrzebna na ustabilizowanie sie pradu przed jego 
					//porownaniem z zadanymi parametrami (np minimalnym pradem ladowania)
					//wielkosc tej zwolki okresla C_zwloka_przed_pomiarem
byte licznik_bledu; 
word PWMlck;
word PWMI;
word PWMU;
//zmienne dot baterii
volatile word ILad;	//surowy prad
volatile word ILadpom;	
volatile word IRozlad; //surowy prad
volatile word IRozladpom; 
volatile word UBat;  //surowe napiecie
volatile word UBatpom;
word UZadane; //zadane napiecie koncowe
volatile word UBatcal; //przeliczone napiecie
volatile word ILadcal; //przeliczony prad
volatile word IRozladcal; //przeliczony prad
word ILadzadany; //zadany prad ladowania
word IRozladzadany; //zadany prad rozladowania
word Usrednie60; //srednie napiecie z minuty pracy
word Isrednie60; //sredni prad z minuty pracy
word ILadcal_filtr2; //przeliczony prad z dodatkowym calkowaniem
word Ifiltrpom;
word deltau;
word deltai;
word pom1;
word pom_C_min_prad_ladowania;
word czas;
long4 pojemnosc; //przechowuje zmierzona pojemnosc baterii Ah * 10 000 000
word pojemnoscwysw; //przechowuje pojemnosc w Ah * 100
long4 pom2;



//definicja typu wyliczeniowego
enum typtrybpracy {laduj200mA, laduj500mA, laduj1000mA, testuj200mA, testuj500mA, testuj1000mA, ostatniapozycja};
//deklaracja zmiennej o zdefiniowanym wlasnie typie wyliczeniowym
enum typtrybpracy trybpracy=laduj200mA;

//definicja drugiego typu wyliczeniowego
enum typfazapracy {ladowanieprzed, rozladowanie, ladowaniepo, koniec, koniec2, blad};
enum typfazapracy fazapracy;

//generator znakow (tablica wartosci bitow na pinach wyswietlacza dla poszczegolnych znakow)
byte generator_znakow[] PROGMEM = {
//BdCAFGED - segmenty wyswietlacza
0b01000100 , //0
0b01011111 , //1
0b01101000 , //2
0b01001010 , //3
0b01010011 , //4
0b11000010 , //5
0b11000000 , //6
0b01001111 , //7
0b01000000 , //8
0b01000010 , //9
0b01000001 , //A
0b11010000 , //b
0b11100100 , //C
0b01011000 , //d
0b11100000 , //E
0b11100001 , //F
0b11011001 , //n
0b11110000 , //t
0b11111001 , //r
};

//komunikaty
char NAPIS_wybrano_tryb[] PROGMEM = "Wybrano tryb: ";
char NAPIS_U_koncowe_rozladowania[] PROGMEM = "U koncowe rozladowania: ";

char NAPIS_blad0[] PROGMEM ="!-----------------------------------!\r\n"
							"Wystapil blad podczas pracy ladowarki\r\n"
							"Prad szybko spadl ponizej ";
char NAPIS_blad1[] PROGMEM = "Sprawdz podlaczenie akumulatora\r\n";

char NAPIS_powitanie[] PROGMEM = "\r\n\r\nWlaczenie zasilania\r\nLadowarka Li-Ion gotowa do pracy\r\n";

char NAPIS_koniec[] PROGMEM = "Proces zakonczony\r\nNapiecie akumulatora bedzie teraz utrzymywane na poziomie ";
char NAPIS_info_tabela[] PROGMEM = "Ponizej beda podawane parametry w tabeli\r\nCzas[min] Napiecie[V] Prad[A]\r\n";
char NAPIS_poczatek_rozladowania[] PROGMEM = "Poczatek rozladowania - - - - - - - - - - - - - - - \r\n";

//poczatek glownej funkcji
int main (void){
 
	//timer2
	TCCR2 |= (1<<WGM21); //CTC
	TCCR2 |= (1<<CS21) ; // (1<<CS00); //presk 8
	OCR2 = 188; //co 125us
	TIMSK |= (1<<OCIE2); //wlaczenie przerwania 
	sei(); //zezwolenie na przerwania

	//adc
	ADMUX |= (1<<REFS1) | (1<<REFS0);   //aref wewn + zewn C 2.56V
	ADCSRA |= (1<<ADEN); //wlaczenie przetwornika
	ADCSRA |= (1<<ADFR); //free run mode
	ADCSRA |= (1<<ADSC); //start konwersji
	ADCSRA |= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0); //prescaler 128

	//ustawienie portow do kontroli PWM jako wyjscia (prad rozladowania i napiecie ladowania)
	DDRC |= (1<<PC2); //PWM I rozlad
	DDRC |= (1<<PC1); //PWM u ladowania

	//odpalenie watchdoga
	wdt_reset();
	wdt_enable(WDTO_500MS);
	
	//init uart
 	USART_Init(__UBRR);
	
	//sprawdzenie czy wlaczono z wcisnietym przyciskiem, jesli tak to wejscie w proc. kalibracji
	_delay_ms(100);
	if (bityA.sw) {
			bityA.sw=0;
			kalibracja();
		}		
	
	//inicjalizacja 
	PrintBuf_PROGMEM(NAPIS_powitanie);
	
	etimer1=1;
	timer_zmiany=0; 
	bityA.migajwysw=0;
		
	//glowna petla
	while(1) {
		
		//watchdog
		if (bityA.watch) {
			bityA.watch=0;
			wdt_reset();
		}


		//obsluga zdarzenia- zmiany trybu pracy 
		if (!timer_zmiany){
			timer_zmiany++;
			PWMI=0;
			PWMU=C_PWMUstart;
			fazapracy=ladowanieprzed;
			bityA.migajled3=0;//niemiaganie diody
			bityA.migajwysw=0;//niemiganie wyswietlacza
			timer_zwloka=C_zwloka_przed_pomiarem;
			UZadane=C_max_napiecie_ladowania;
			licznik_bledu=0;
			etimer2=0;
			Usrednie60=0;
			Isrednie60=0;
			ILadcal_filtr2=0;
			Ifiltrpom=0;
			deltau=C_deltau;
			czas=0;
			bityA.flaga_pom=1;
			switch (trybpracy)		{
				case laduj1000mA:
						wysw_liczbe(10); 
						wyswb2=pgm_read_byte(&generator_znakow[12]); //litera C
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=100; 
						IRozladzadany=0;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Laduj 1000mA\r\n");
						podaj_parametry_laduj();
				break;
				case laduj500mA:
						wysw_liczbe(5); 
						wyswb2=pgm_read_byte(&generator_znakow[12]); //litera C
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=50; 
						IRozladzadany=0;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Laduj 500mA\r\n");
						podaj_parametry_laduj();
				break;
				case laduj200mA:
						wysw_liczbe(2); 
						wyswb2=pgm_read_byte(&generator_znakow[12]); //litera C
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=20; 
						IRozladzadany=0;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania200;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Laduj 200mA\r\n");
						podaj_parametry_laduj();
				break;
				case testuj1000mA:
						wysw_liczbe(10); 
						wyswb2=pgm_read_byte(&generator_znakow[17]); //litera t
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=0; 
						IRozladzadany=100;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Testuj 1000mA\r\n");
						podaj_parametry_testuj();

				break;
				case testuj500mA:
						wysw_liczbe(5); 
						wyswb2=pgm_read_byte(&generator_znakow[17]); //litera t
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=0; 
						IRozladzadany=50;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Testuj 500mA\r\n");
						podaj_parametry_testuj();
				break;
				case testuj200mA:
						wysw_liczbe(2); 
						wyswb2=pgm_read_byte(&generator_znakow[17]); //litera t
						wyswb1 &= ~(1<<6); //zapalenie kropki
						//ustawienie parametrow procesu
						ILadzadany=0; 
						IRozladzadany=20;
						bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						pom_C_min_prad_ladowania=C_min_prad_ladowania200;
						PrintBuf("\r\n");
						PrintBuf_PROGMEM(NAPIS_wybrano_tryb); //wyslanie uartem napisu umieszczonego w pamieci flash
						PrintBuf("Testuj 200mA\r\n");
						podaj_parametry_testuj();
				break;
				case ostatniapozycja:
					trybpracy=0;
					timer_zmiany=0;
				break; 
			}
		}


		//stablizacja pradu / napiecia i odswiezanie wysw (co sekunde) oraz zmiany fazy pracy
		if (etimer3) {
			etimer3=0;
			if (timer_zwloka) timer_zwloka--;

			//obsluga wyswietlania fazy pracy, obliczen, oraz zmiany na kolejna faze
			if (timer_zmiany>100) {
				switch (fazapracy){
					case ladowanieprzed:
						//wyswietlanie co minute parametrow procesu
						if (etimer2==C_minuta) {
							etimer2=0;
							czas++;
							Usrednie60/=C_minuta;
							Isrednie60/=C_minuta;
							PrintBuf_LICZBA_WORD(czas,0);
							PrintbinBuf(9);
							PrintBuf_LICZBA_WORD(Usrednie60,3);
							PrintbinBuf(9);
							if (Isrednie60<100) PrintBuf("0");
							PrintBuf_LICZBA_WORD(Isrednie60,3);
							PrintBuf("\r\n");
							Usrednie60=0;
							Isrednie60=0;
						}
						Usrednie60+=UBatcal;
						Isrednie60+=ILadcal;
						
						//jesli tryb pracy ustawiony na test to wlaczenie najpierw pradu ladowania
						if ( trybpracy > laduj1000mA ) {
							if (IRozladzadany) {
								PWMU=C_PWMUstart;
								ILadzadany=IRozladzadany;
								IRozladzadany=0;
							}
							//--------------------------------------------------------------
							//jesli juz sie skonczylo ladowac przed rozladowaniem
							if ( (ILadcal_filtr2 <= pom_C_min_prad_ladowania) && !timer_zwloka) {
								if (licznik_bledu > 1) {
									IRozladzadany=ILadzadany;
									ILadzadany=0;
									fazapracy=rozladowanie;
									licznik_bledu=0;
									pojemnosc=0;
									etimer2=0;
									czas=0;
									Usrednie60=0;
									Isrednie60=0;
									PrintBuf_PROGMEM(NAPIS_poczatek_rozladowania);
									}
								licznik_bledu++;
								if (ILadcal <= C_min_prad_awaria) fazapracy=blad;
								
							}
						}
						else
						//----------------------------------------------------------
						//jesli tryb zwykly to badanie czy ladowanie dobieglo konca
						{
							if ( (ILadcal_filtr2 <= pom_C_min_prad_ladowania) && !timer_zwloka) {
								if (licznik_bledu > 1) {
									fazapracy=koniec2;
									}
								licznik_bledu++;
								if (ILadcal <= C_min_prad_awaria) fazapracy=blad;
							}
						
						}
						if (!wyswietlanie) {
							//wyswietlanie napiecia baterii
							wyswietlanie=1;
							wysw_liczbe(UBatcal);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=1 ; bityA.led3=0; bityA.led2=0; //zapalenie led2 U
						}
						else {
							//wyswietlanie pradu ladowania
							wysw_liczbe(ILadcal_filtr2);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							wyswietlanie=0;
							bityA.led1=0 ; bityA.led3=0; bityA.led2=1; //zapalenie led2 I
						}	
						 
					break;
					case rozladowanie:
						//wyswietlanie co minute parametrow procesu
						if (etimer2==C_minuta) {
							etimer2=0;
							czas++;
							Usrednie60/=C_minuta;
							Isrednie60/=C_minuta;
							PrintBuf_LICZBA_WORD(czas,0);
							PrintbinBuf(9);
							PrintBuf_LICZBA_WORD(Usrednie60,3);
							PrintbinBuf(9);
							if (Isrednie60<100) PrintBuf("0");
							PrintBuf_LICZBA_WORD(Isrednie60,3);
							PrintBuf("\r\n");
							Usrednie60=0;
							Isrednie60=0;
						}
						Usrednie60+=UBatcal;
						Isrednie60+=IRozladcal;
						switch (wyswietlanie){
						case 1:							
							//badanie, czy rozladowanie dobieglo konca
							if ( (UBatcal <= C_min_napiecie_rozladowania) ) {
								ILadzadany=IRozladzadany;
								IRozladzadany=0;
								fazapracy=ladowaniepo;
								timer_zwloka=C_zwloka_przed_pomiarem;	
								etimer2=0;
								czas=0;
								deltau=C_deltau;
								PWMU=C_PWMUstart;
								Usrednie60=0;
								Isrednie60=0;
								PrintBuf("Ladowanie po pomiarze pojemnosci - - - - - - - - -\r\n");
							}
							//wyswietlanie napiecia baterii
							wysw_liczbe(UBatcal);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=1 ; bityA.led2=0; //zapalenie led2 U
							bityA.led3=0 ;
							bityA.migajled3=0;
							bityA.migajwysw=0;//niemiganie wyswietlacza
						break;
						case 2:
							//wyswietlanie pradu rozladowania
							wysw_liczbe(IRozladcal);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=0 ; bityA.led2=1; //zapalenie led2 I
						break;
						case 3:
							//wyswietlanie wyladowanej pojemnosci z miganiem (bo nie koniec procesu)
							wysw_liczbe(pojemnoscwysw);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=0 ; bityA.led2=0; 
							bityA.migajled3=1;	//miganie led3 (Ah)
							bityA.migajwysw=1;//miganie wyswietlacza
						break;
						case 4:
							wyswietlanie=0;
						break;
						}
						wyswietlanie++;
						//obliczanie wyssanej z akumulatora pojemnosci (doliczane co sekunde)
						pom2=IRozladcal * 1000000;
						pojemnosc += (pom2 / 3600);
						pojemnoscwysw=pojemnosc /   1000000;
						 	
					break;
					case ladowaniepo:
						//wyswietlanie co minute parametrow procesu
						if (etimer2==C_minuta) {
							etimer2=0;
							czas++;
							Usrednie60/=C_minuta;
							Isrednie60/=C_minuta;
							PrintBuf_LICZBA_WORD(czas,0);
							PrintbinBuf(9);
							PrintBuf_LICZBA_WORD(Usrednie60,3);
							PrintbinBuf(9);
							if (Isrednie60<100) PrintBuf("0");
							PrintBuf_LICZBA_WORD(Isrednie60,3);
							PrintBuf("\r\n");
							Usrednie60=0;
							Isrednie60=0;
						}
						Usrednie60+=UBatcal;
						Isrednie60+=ILadcal;

						switch (wyswietlanie){
						case 1:							
							//jesli ladowanie dobieglo konca
							if ( (ILadcal_filtr2 <= pom_C_min_prad_ladowania) && !timer_zwloka) {
								if (licznik_bledu > 1) {
									fazapracy=koniec;
									}
								licznik_bledu++;
								if (ILadcal <= C_min_prad_awaria) fazapracy=blad;
							}
							//wyswietlanie napiecia baterii
							wysw_liczbe(UBatcal);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=1 ; bityA.led3=0; //zapalenie led2 U
						break;
						case 2:
							//wyswietlanie pradu ladowania
							wysw_liczbe(ILadcal_filtr2);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led1=0 ; bityA.led2=1; //zapalenie led2 I
						break;
						case 3:
							//wyswietlanie wyladowanej, zmierzonej juz pojemnosci
							wysw_liczbe(pojemnoscwysw);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led2=0 ; bityA.led3=1; //zapalenie led3 Ah
						break;
						case 4:
							wyswietlanie=0;
						break;
						}
						wyswietlanie++;
					break;
					//wyswietlanie napisu End oraz zmierzonej pojemnosci
					case koniec:
						if (bityA.flaga_pom){
							//wyslanie napisu uartem
							PrintBuf_PROGMEM(NAPIS_koniec);
							PrintBuf_LICZBA_WORD(C_napiecie_doladowania,3);
							PrintBuf("V\r\n");
						}
						bityA.flaga_pom=0;
						if (!wyswietlanie) {
							//wyswietlanie napisu END
							wyswietlanie=1;
							wyswb2=pgm_read_byte(&generator_znakow[14]); //E
							wyswb1=pgm_read_byte(&generator_znakow[16]); //n
							wyswb0=pgm_read_byte(&generator_znakow[13]); //d
							bityA.led1=0 ; bityA.led3=0; bityA.led2=0; //zgaszenie ledow
							//ustawienie nizszego napiecia na zaciskach baterii
							UZadane=C_napiecie_doladowania;
							//ustawienie pradu po zakonczeniu ladowania (w normalnych warunkach ten prad nie jest nigdy osiagany)
							ILadzadany=C_prad_doladowania; 
							IRozladzadany=0;
						}
						else {
							//wyswietlanie wyladowanej, zmierzonej juz pojemnosci
							wysw_liczbe(pojemnoscwysw);
							wyswb2 &= ~(1<<6); //zapalenie kropki
							bityA.led2=0 ; bityA.led3=1; //zapalenie led3 Ah
							wyswietlanie=0;
						}	
					break;
					//wyswietlanie napisu End
					case koniec2:
						if (bityA.flaga_pom){
							//wyslanie napisu uartem
							PrintBuf_PROGMEM(NAPIS_koniec);
							PrintBuf_LICZBA_WORD(C_napiecie_doladowania,3);
							PrintBuf("V\r\n");
						}
						bityA.flaga_pom=0;
						//wyswietlanie napisu END
						wyswb2=pgm_read_byte(&generator_znakow[14]); //E
						wyswb1=pgm_read_byte(&generator_znakow[16]); //n
						wyswb0=pgm_read_byte(&generator_znakow[13]); //d
						bityA.led1=0 ; bityA.led3=0; bityA.led2=0; //zgaszenie ledow
						//ustawienie pradu po zakonczeniu ladowania (w normalnych warunkach ten prad nie jest nigdy osiagany)
						ILadzadany=C_prad_doladowania; 
						IRozladzadany=0;
						//ustawienie nizszego napiecia na zaciskach baterii
						UZadane=C_napiecie_doladowania;
					break;
					//wyswietlanie napisu Err
					case blad:
						if (bityA.flaga_pom){
							//wyslanie napisu uartem
							PrintBuf_PROGMEM(NAPIS_blad0);
							PrintBuf(xstr(C_min_prad_awaria));
							PrintBuf("0mA\r\n");
							PrintBuf_PROGMEM(NAPIS_blad1);
						}
						bityA.flaga_pom=0;
						wyswb2=pgm_read_byte(&generator_znakow[14]); //E
						wyswb1=pgm_read_byte(&generator_znakow[18]); //r
						wyswb0=pgm_read_byte(&generator_znakow[18]); //r
						//ustawienie pradu po zakonczeniu ladowania (w normalnych warunkach ten prad nie jest nigdy osiagany)
						ILadzadany=C_prad_doladowania; 
						IRozladzadany=0;
						//ustawienie nizszego napiecia na zaciskach baterii
						UZadane=C_napiecie_doladowania;
						//miganie led 2
						if (!wyswietlanie) {
							wyswietlanie=1;
							bityA.led1=0 ; bityA.led2=0; bityA.led3=0;
						}
						else {
							wyswietlanie=0;
							 bityA.led2=1;
						}	

					break;


				}
			}



			deltai=IRozladzadany-IRozladcal;
			if (deltai<10) deltai=1;
			if (deltai>50) deltai=50;
			//jesli prad wiekszy niz zadany to zmniejszenie wsp wypelnienia PWM
			if ( ( IRozladcal > IRozladzadany ) && PWMI)  PWMI--; 
			else  PWMI+=deltai; //jesli nie to zwiekszenie wsp wypelnienia PWM

			
			//stablilizacja pradu ladowania oraz napiecia ladowania
			
			if (deltau>1) deltau--;

			//jesli prad ladownia lub napiecie wieksze niz zadane to zmniejszenie wsp wypelnienia PWM U Ladowania
			if ( ( (ILadcal > ILadzadany) || (UBatcal > UZadane) ) && PWMU) {
				if (PWMU >  deltau) PWMU-=deltau; 
			}
			else  {
				PWMU+=deltau; 
			}
	
			//blokada PWM rozladowania
			if (!IRozladzadany) PWMI=0;

			//blokada PWM U ladowania
			if (!ILadzadany) PWMU=0;

			if (PWMU>512) PWMU=512;
			if (PWMI>512) PWMI=512;

			//dodatkowe calkowanie wielkosci pradu
			Ifiltrpom += ILadcal; 
			Ifiltrpom -= ILadcal_filtr2;
			ILadcal_filtr2 = Ifiltrpom / 10;


		}
		


		//obsluga PWM rozladowania
		PWMlck++;
		if (PWMlck == 512) PWMlck=0;
		if (PWMlck > PWMI) PWMIRozladOff; else PWMIRozladOn;
		//obsluga PWM napiecia ladowania
		if (PWMlck > PWMU) PWMULadOff; else PWMULadOn;


		//jesli wcisnieto przycisk - zmiana trybu pracy
		if (bityA.sw) {
			bityA.sw=0; //skasowanie flagi oznaczajacej, ze przycisk byl wcisniety
			trybpracy++; //przejscie do nastpenej pozycji w menu
			timer_zmiany=0;
			etimer1=1;
	
		}		
	
	
	}


}
//koniec glownej funkcji--------------------------------------

//przerwanie czasu
ISR (TIMER2_COMP_vect){
	etimer1++;
	word pom4w;	

	//co sekunde (liczenie czasu)
	if (etimer1==8000) {
		etimer1=0;
		//sekundnik stablizacji PWM
		etimer3++;
		//czas procesu
		etimer2++;
	}
	
	//co 32ms (odczyty A/C) oraz timer_zmiany (czasy wyswietlania napisow itp)
	if (!((byte)etimer1)){
		bityA.watch=1;
		//licznik migania led itp
		etimer4++;

		timer_zmiany++;
		if(!timer_zmiany) timer_zmiany=0xFF;
		etimeradc++;
		//Przeliczenie wartosci z przetwornika na wartosci skalibrowane
		UBatcal = (UBat >> 1); //podzial przez 2
		IRozladcal= (IRozlad >> 3); //podzial przez 8
		ILadcal= (ILad >> 3); //podzial przez 8
		if (ILadcal >= C_kalibracja_korekta_pradu_ladowania) ILadcal-= C_kalibracja_korekta_pradu_ladowania; 
		else ILadcal=0;

	}
	
	switch (etimeradc){
		case 2:
		
			//przygotowanie do pomiaru U
			//przelaczenie by przetwornik mierzyl UBat
			ADMUX &= 0b11110000;
			ADMUX |= PinPomiarUBat;
		break;
		case 4:
		
			pom4w = (word)ADCL; //pomiar napiecia UBat
			pom4w |=  ((word)ADCH<<8);
			UBatpom += pom4w; //filtr dolnoprzepustowy 1 rzedu
			UBatpom -= UBat;
			UBat = UBatpom / C_stala_czasowa;	//Ubat zawiera surowe dane z przetwornika ale juz po filtrze LP
			etimeradc++;
		break;
		case 5:
			//przygotowanie do pomiaru Ilad
			//przelaczenie by przetwornik mierzyl Ilad
			ADMUX &= 0b11110000;
			ADMUX |= PinPomiarILad;
		break;
		case 7:
			pom4w = (word)ADCL; //pomiar ILad
			pom4w |=  ((word)ADCH<<8);
			ILadpom += pom4w; //filtr dolnoprzepustowy 1 rzedu
			ILadpom -= ILad;
			ILad = ILadpom / C_stala_czasowa;	//ILad zawiera surowe dane z przetwornika ale juz po filtrze LP
			etimeradc++;
		break;
		case 8:
			//przygotowanie do pomiaru IRozlad
			//przelaczenie by przetwornik mierzyl IRozlad
			ADMUX &= 0b11110000;
			ADMUX |= PinPomiarIRozlad;
		break;
		case 10:
			pom4w = (word)ADCL; //pomiar IRozlad
			pom4w |=  ((word)ADCH<<8);
			IRozladpom += pom4w; //filtr dolnoprzepustowy 1 rzedu
			IRozladpom -= IRozlad;
			IRozlad = IRozladpom / C_stala_czasowa;	//IRozlad zawiera surowe dane z przetwornika ale juz po filtrze LP
			etimeradc=0;
		break;
	}

	

	//odswiezanie wyswietlacza i klawiatury 
	odwysw() ;


}



//ukazuje liczbe na wyswietlaczu
void wysw_liczbe(word param_1){
	word pom4w;
	//setki
	pom4w = param_1 / 100;
	wyswb2=pgm_read_byte(&generator_znakow[pom4w]); //zapis odpowiedniego bajtu do bufora wysw.
	//dziesiatki
	pom4w = (param_1 % 100) / 10;
	wyswb1=pgm_read_byte(&generator_znakow[pom4w]); //zapis odpowiedniego bajtu do bufora wysw.
	//jednosci
	pom4w = param_1 % 10;
	wyswb0=pgm_read_byte(&generator_znakow[pom4w]); //zapis odpowiedniego bajtu do bufora wysw.
}

//funkcja wysyla ciag z pamieci flash
void PrintBuf_PROGMEM(char * str) {
	register char znak;
	while ( (znak=pgm_read_byte(str++)) ) PrintbinBuf(znak);
}

//funkcja wysyla uartem liczbe zamieniona na ciag, kropka oznacza miejsce po ktorej cyfrze zostanie
//wstawiona kropka
void PrintBuf_LICZBA_WORD(word liczba, byte kropka){
	byte pom4;
	byte flaga=0;
	pom4= liczba/10000;
	if (pom4) { PrintbinBuf(pom4+48) ; flaga=1; }
	if (kropka==1) PrintbinBuf(46);	

	pom4= (liczba % 10000) / 1000;
	if (pom4 || flaga) { PrintbinBuf(pom4+48) ; flaga=1; }
	if (kropka==2) PrintbinBuf(46);	

	pom4= (liczba % 1000) / 100;
	if (pom4 || flaga) { PrintbinBuf(pom4+48) ; flaga=1; }
	if (kropka==3) PrintbinBuf(46);		

	pom4= (liczba % 100) / 10;
	if (pom4 || flaga) { PrintbinBuf(pom4+48) ; flaga=1; }
	if (kropka==4) PrintbinBuf(46);	

	pom4= liczba % 10;
	PrintbinBuf(pom4+48);
}

//wysyla uartem parametry testowania akumulatora
void podaj_parametry_testuj(void){
	PrintBuf_PROGMEM(NAPIS_U_koncowe_rozladowania);
	PrintBuf_LICZBA_WORD(C_min_napiecie_rozladowania,3);
	PrintBuf("V\r\n");

	PrintBuf("I Rozladowania: ");
	PrintBuf_LICZBA_WORD(IRozladzadany,0);
	PrintBuf("0mA\r\n");

	PrintBuf("U Ladowania: ");
	PrintBuf_LICZBA_WORD(C_max_napiecie_ladowania,3);
	PrintBuf("V\r\n");

	PrintBuf("I Ladowania: ");
	PrintBuf_LICZBA_WORD(IRozladzadany,0);
	PrintBuf("0mA\r\n");

	PrintBuf("I koncowy ladowania: ");
	PrintBuf_LICZBA_WORD(pom_C_min_prad_ladowania,0);
	PrintBuf("0mA\r\n");


	PrintBuf("U Podtrzymania: ");
	PrintBuf_LICZBA_WORD(C_napiecie_doladowania,3);
	PrintBuf("V\r\n");

	PrintBuf("Fazy pracy: Ladowanie > Rozladowanie > Ladowanie\r\n");
	PrintBuf_PROGMEM(NAPIS_info_tabela);
}

//wysyla uartem parametry ladowania akumulatora
//wysyla uartem parametry testowania akumulatora
void podaj_parametry_laduj(void){

	PrintBuf("U Ladowania: ");
	PrintBuf_LICZBA_WORD(C_max_napiecie_ladowania,3);
	PrintBuf("V\r\n");

	PrintBuf("I Ladowania: ");
	PrintBuf_LICZBA_WORD(ILadzadany,0);
	PrintBuf("0mA\r\n");

	PrintBuf("I koncowy ladowania: ");
	PrintBuf_LICZBA_WORD(pom_C_min_prad_ladowania,0);
	PrintBuf("0mA\r\n");

	PrintBuf("U Podtrzymania: ");
	PrintBuf_LICZBA_WORD(C_napiecie_doladowania,3);
	PrintBuf("V\r\n");

	PrintBuf_PROGMEM(NAPIS_info_tabela);
}


//procedura uruchamia sie w chwili wlaczenia urzadzenia z wcisnietym przyciskiem
//odpowiednie odczyty napiec i pradow ustawia sie potencjometrami montazowymi
void kalibracja(void){
	byte lpom1=0;
	byte lpom2=0;
	while(1){
		wdt_reset();
		//PWM - sterowanie pradem rozladowania
		lpom1++;
		if ( lpom1>6 ) PWMIRozladOn; else PWMIRozladOff;
		if ( lpom1==10) lpom1=0;
		//Ustawienie Ulad= max;
		PWMULadOn;
		if (bityA.sw) {
			bityA.sw=0;
			lpom2++;
		}		
		switch (lpom2){
			case 0:
	 			//ustawianie odczytu napiecia
				//swieci led U
				//zmierzyc napiecie miedzy + a - baterii oraz ustawic odpowiedni odczyt PR1
				//|-----------------------|
				//|                       | 
				//|             R3        | 
				//|                       | 
				//|           PR2         | PR2 - pomiar I Lad
				//|                   PR1 | PR1 - pomiar U Bat (u ladowania)
				//|        PR3 R4         | PR3 - pomiar I Rozlad, 
				//|-----------------------|  
				bityA.led2=0; 
				bityA.led1=1; 
				wysw_liczbe(UBatcal);
			break;
			case 1:
				//ustawianie odczytu pradu rozladowania
				//zmierzyc spadek napiecia na R3 (1R 1W) i ustawic odczyt PR3
				//swieca dwa ledy
				bityA.led2=1; 
				wysw_liczbe(IRozladcal);
			break;
			case 2:
				//ustawianie odczytu pradu ladowania
				//ustawic taki sam odczyt jak dla pradu rozladowania (regulowac PR2)
				//swieci led I
				bityA.led1=0; 
				bityA.led2=1; 
				wysw_liczbe(ILadcal);
			break;
			//druga metoda kalibracji pomiaru pradu(dokladniejsza):
			//1. wlaczyc ladowarke w trybie ladowania 0,5A i zmierzyc prad plynacy przez baterie
			//po czym ustawic odpowiedni odczyt PR2
			//2. wejsc w tryb kalibracji i zrownac ze soba odczyty I Lad i I Rozlad regulujac PR3
			//


		}
 		if (lpom2>2) lpom2=0;
	}
}


